/*******************************************************************************

  AMPERIMETRO P/ 20 AMP (DC)

  C/ PIC16F1220 E ACS712-20A

  Autor: Cludio Lrios

  Data: 25/02/2019

  Descrio: Amperimetro de -20 a +20 Amp DC usando o integrado ACS712T-ELC20A
  como sensor de corrente
  Leituras de corrente contnua (DC)
  
  Blog Picsource

  Uso didtico
*******************************************************************************/
//Descomente apenas um modelo e comente os outros 2
//#define ASC712_05A
#define ASC712_20A    //escolhido modelo para 20 Amperes
//#define ASC712_30A


#include <AMPERIMETER_1220.h>
#define use_portb_lcd
#include <LCD.C>
#byte adcon0 = 0xfc2

//variveis globais
float val_corrente;
char sinal;
unsigned int8 a;
unsigned int16 res;

//defines
#define V_VCC  5000  //tenso do vcc do pic (mV).

#ifdef ASC712_05A
 #define VAL_SENSITIVITY 185.0 // sensibilidade de 185 mV/A
#endif


#ifdef ASC712_20A
 #define VAL_SENSITIVITY 100.0 // sensibilidade de 100 mV/A
#endif

#ifdef ASC712_30A
 #define VAL_SENSITIVITY 66.0 // sensibilidade de 66 mV/A
#endif
/*******************************************************************************
             MAIN (CONFIGURAES)
*******************************************************************************/
void main() {

   setup_oscillator(OSC_4MHZ|OSC_INTRC);
   setup_adc_ports(sAN0);//seleciona pino 1 como entrada do ADC
   setup_adc(ADC_CLOCK_INTERNAL|ADC_TAD_MUL_6);
   setup_wdt(WDT_OFF);
   setup_timer_0(RTCC_INTERNAL);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_timer_3(T3_DISABLED|T3_DIV_BY_1);
   adcon0&=0b00111111; //ref interna vdd |vss
   lcd_init(); //inicializa LCD
   lcd_gotoxy(3,1);
   printf(lcd_putc,"Amperimetro");//mostra o nome
   lcd_gotoxy(3,2);
   printf(lcd_putc, "by Picsource");//mostra o blog
   delay_ms(2000);//delay de apresentao
/*******************************************************************************
             LOOP DE CONTINUA REPETIO DO PROGRAMA
*******************************************************************************/
   for(;;){
     set_adc_channel(0);//pino 1 como entrada
     delay_us(10);//delay mnimo
     res=0;
     for(a=0;a<64;a++){//64 amostragens
     res += read_adc();//soma leituras sucessivas
     delay_us(100);
     }
     res=res/64;//faz a mdia das 64 converses

     //ajusta resultado para indicar como leitura positiva ou negativa

     if(res >= 512) {//se for maior ou igual a 512  medida positiva de corrente
       res = res-512;
       sinal = ' ';//sinal para positivo (' ' ou '+')
     }
     else{//se for menor que 512  medida negativa de corrente
       res= 512 - res;
       sinal='-';//sinal negativo
     }
  
     val_corrente= (float)res * V_VCC / (1023.0 * VAL_SENSITIVITY);//calcula valor da corrente
     
     lcd_gotoxy(3,2);//aponta onde deve ser impresso
     printf(lcd_putc, "I =  %02.2f A   ",val_corrente);//imprime resultado no LCD
     lcd_gotoxy(7,2);//aponta o local do sinal
     printf(lcd_putc, "%c",sinal);// mostra o sinal da corrente medida
     delay_ms(500);//tempo entre apresentaes
   }//for(;;)

}//main()
